//---------------------------------------------------------------
// Events
//---------------------------------------------------------------

addEventListener("getOfflineBookings", async (resolve, reject, args) => {
  try {
    const result = CapacitorKV.get("offline_buchungen");
    console.log("result of offline booking: " + JSON.stringify(result));
    if (result == undefined || result == null) {
      resolve();
      return;
    }
    resolve(result);
  } catch (err) {
    console.error(err);
    console.error(JSON.stringify(err));
    reject(err);
  }
});

addEventListener("setOfflineBookings", async (resolve, reject, args) => {
  try {
    CapacitorKV.set("offline_buchungen", args.of);
    resolve();
  } catch (err) {
    console.error(err);
    console.error(JSON.stringify(err));
    reject(err);
  }
});

addEventListener("removeOfflineBookings", async (resolve, reject, args) => {
  try {
    CapacitorKV.remove("offline_buchungen");
    resolve();
  } catch (err) {
    console.error(err);
    console.error(JSON.stringify(err));
    reject(err);
  }
});

addEventListener("sendOfflineBookings", async (resolve, reject, args) => {

  try {
    let debug = false;

    //----------------------------------------------------
    // Network Status
    //----------------------------------------------------

    // OHNE AWAIT!!
    const status = CapacitorDevice.getNetworkStatus();

    if (!status.connected) {
      console.log("keine Verbindung zum Netzwerk");
      resolve();
      return;
    }

    //bookingCodeShortName -> booking_code

    //----------------------------------------------------
    // Offline Buchungen abrufen
    //----------------------------------------------------
    const response = CapacitorKV.get("offline_buchungen");
    if (response == undefined || response == null) {
      console.log("keine Offline Buchungen im Speicher");
      resolve();
      return;
    }

    let arrBookings = JSON.parse(response.value);

    //----------------------------------------------------
    // Request bauen
    //----------------------------------------------------

    const body = createPostBody(arrBookings);

    if (body == false) {
      console.log("body konnte nicht gebaut werden");
      resolve();
      return;
    }

    const hash = CapacitorKV.get("hash");
    const deviceId = CapacitorKV.get("deviceId");
    if (hash == undefined || deviceId == undefined) {
      console.log("headers konnte nicht gebaut werden - session nicht gesetzt?");
      resolve();
      return;
    }

    const options = {
      method: "POST",
      headers: {
        "Content-Type": "application/json",
        "Pz-Device-Id": deviceId.value,
        "Pz-Session-Token": hash.value
      },
      body: JSON.stringify(body),
    };

    if (debug) {
      console.log("REQUEST STRING - options");
      console.log(JSON.stringify(options));
    }

    const host = CapacitorKV.get("host");

    if (debug) {
      console.log("HOST");
      console.log(host.value);
    }

    //----------------------------------------------------
    // Request bauen
    //----------------------------------------------------
    const path = '/api/v2/time_records/offline';
    const url = buildURL(host.value, path);
    fetch(url, options)
      .then(httpResponse => httpResponse.json())
      .then((httpResponse) => {
        if (debug) {
          console.log("ERGEBNIS DER HTTP REQUEST");
          console.log(httpResponse);
          console.log(JSON.stringify(httpResponse));
          console.log(httpResponse.value);

          if (Array.isArray(httpResponse.value)) {
            console.log("ist array");
          }
        }

        arrBookings.forEach(el => {
          if (httpResponse.value.find((element) => element == el.id)) {
            el.gesendet = true;
          }
        });

        CapacitorKV.set("offline_buchungen", JSON.stringify(arrBookings));

        resolve();
      })
      .catch((err) => {
        console.log("Es ist ein Fehler beim FETCH aufgetreten");
        console.log(err);
        reject();
      });

  } catch (err) {
    console.log(err);
    reject();
  }
});

addEventListener("setSession", async (resolve, reject, args) => {
  try {
    CapacitorKV.set("hash", args.hash);
    CapacitorKV.set("deviceId", args.deviceId);
    CapacitorKV.set("host", args.host);
    resolve();
  } catch (err) {
    console.error(err);
    console.error(JSON.stringify(err));
    reject(err);
  }
});

//---------------------------------------------------------------
// Test-Events
//---------------------------------------------------------------

//Macht ein GET-Request welcher die IP des Gateways liefert und gibt es in der Console aus
addEventListener("icanhazip", async (resolve, reject, args) => {
  try {
    const options = {
      method: "GET",
      headers: {},
    };

    if (false) {
      fetch("https://icanhazip.com", options)
        .then((response) => response.text())
        .then((response) => {
          console.log(response);
          resolve();
        })
        .catch((err) => {
          console.log(err);
          resolve();
        });
    }

    if (true) {
      const res = await fetch("https://icanhazip.com/");

      if (!res.ok) {
        console.error("Network response was not OK");
      } else {
        console.log("Network response was OK");

        const text = await res.json();

        console.log(text);
      }

      // fetch("https://icanhazip.com/").then((response) => response.text()).then((text) => { console.log(text); resolve(); });
    }
  } catch (err) {
    console.error(err);
    console.error(JSON.stringify(err));
    reject(err);
  }
});

//Test-Request zum Senden einer Offline-Buchung an planzeit Web
addEventListener("sendOfflineBookingsStatic", async (resolve, reject, args) => {
  try {
    const options = {
      method: "POST",
      headers: {
        "Content-Type": "application/json",
      },
      body: JSON.stringify({
        buchungscode: "KO",
        hash: "1234",
        zeit: "13:04",
        datum: "2022-03-04",
        device_id: "1234",
        latitude: 5.345454545,
        longitude: 2.45454545,
        action: "app-call",
        real_action: "offline-buchung",
      }),
    };

    fetch("https://local-dev-louis.planzeit-dev.de", options)
      .then((response) => response.text())
      .then((response) => {
        console.log(response);
      })
      .catch((err) => console.log(err));

    resolve();
  } catch (err) {
    console.error(err);
    console.error(JSON.stringify(err));
    reject(err);
  }
});

//---------------------------------------------------------------
// Functions
//! Das JS-Environment ist sehr schwach; daher können keine Imports verwendet werden
//---------------------------------------------------------------

function createPostBody(bookings) {
  let allBookings = JSON.parse(JSON.stringify(bookings));
  allBookings = allBookings.filter((b) => {
    if (b.gesendet) {
      return false;
    }
    b.booking_code = b.bookingCodeShortName;
    return true;
  });

  let body = {
    bookings: allBookings
  };

  return body;
}

function buildURL(host, path) {
  try {
    if (host === '' || host === null) {
      return null;
    }

    if (!host.startsWith("https://")) {
      host = "https://" + host;
    }

    if (host.endsWith("/")) {
      host = host.substr(0, host.length -1);
    }

    return host + path;

    //! Die Klasse URL ist nicht in dieser JavaScript Umgebung implementiert
    //return new URL(prefix, host).href;

  } catch (err) {
    console.log(err);
    return null;
  }
}
